import { HttpClientModule } from '@angular/common/http';
import { FormsModule } from '@angular/forms';
import { BrowserModule } from '@angular/platform-browser';
import { NgModule } from '@angular/core';
import { NgbModule } from '@ng-bootstrap/ng-bootstrap';

import { AppComponent } from './app.component';
import { DashboardComponent } from './dashboard/dashboard.component';
import { HeaderComponent } from './header/header.component';
import { SidebarComponent } from './sidebar/sidebar.component';
import { MarkdownService } from './services/markdown.service';
import { TerminadoService } from './services/terminado.service';
import { IdeComponent } from './ide/ide.component';
import { MessagesComponent } from './messages/messages.component';
import { WebSocketService } from './services/websocket.service';
import { TerminalComponent } from './terminal/terminal.component';
import { AppRoutingModule } from './app-routing.module';
import { TestmessengerComponent } from './testmessenger/testmessenger.component';
import { DeploymentNotificationService } from './services/deployment-notification.service';
import { SafePipe, SafeHtmlPipe } from './pipes/safe.pipe';
import { ConsoleComponent } from './console/console.component';
import { MonacoEditorModule } from 'ngx-monaco-editor';
import {
  ConfigReadyService,
  IdeConfigService,
  DashboardConfigService,
  SiteConfigService,
} from './services/config.service';
import { FSMUpdateService } from './services/fsmupdate.service';
import { LoaderComponent } from './loader/loader.component';


@NgModule({
  declarations: [
    AppComponent,
    HeaderComponent,
    SidebarComponent,
    IdeComponent,
    MessagesComponent,
    TerminalComponent,
    DashboardComponent,
    TestmessengerComponent,
    SafePipe,
    SafeHtmlPipe,
    ConsoleComponent,
    LoaderComponent
  ],
  imports: [
    BrowserModule,
    FormsModule,
    HttpClientModule,
    NgbModule,
    AppRoutingModule,
    MonacoEditorModule.forRoot()
  ],
  providers: [
    MarkdownService,
    WebSocketService,
    TerminadoService,
    DeploymentNotificationService,
    ConfigReadyService,
    IdeConfigService,
    DashboardConfigService,
    SiteConfigService,
    FSMUpdateService
  ],
  bootstrap: [
    AppComponent
  ]
})
export class AppModule {}
